import React, { Component, PropTypes } from 'react';

const ease = require('d3-ease');

class Bar extends Component {
  static defaultProps = {
    height: 0,
    width: 0,
    x: 0,
    y: 0
  };

  static propTypes = {
    animation: PropTypes.shape({
      duration: PropTypes.number.isRequired
    }).isRequired,
    baseLine: PropTypes.number.isRequired,
    fill: PropTypes.string.isRequired,
    height: PropTypes.number.isRequired,
    labels: PropTypes.shape({
      color: PropTypes.string.isRequired,
      digitsAfterPoint: PropTypes.number.isRequired,
      disable: PropTypes.bool.isRequired,
      fontSize: PropTypes.string.isRequired
    }).isRequired,
    value: PropTypes.number.isRequired,
    width: PropTypes.number.isRequired,
    x: PropTypes.number.isRequired,
    y: PropTypes.number.isRequired
  };

  state = {
    height: 0,
    heightCache: 0,
    milliseconds: 0,
    opacity: 0,
    value: 0,
    valueCache: 0,
    y: this.props.baseLine,
    yCache: this.props.baseLine
  };

  componentDidMount() {
    this.interval = requestAnimationFrame(this.tick.bind(this));
  }


  componentWillReceiveProps(nextProp) {
    const { height, value, y } = this.props;

    if (height !== nextProp.height || y !== nextProp.y) {
      this.setState({
        heightCache: height,
        milliseconds: 0,
        valueCache: value,
        yCache: y
      });
    }
  }

  shouldComponentUpdate() {
    return this.props.height !== this.state.height ||
      this.props.y !== this.state.y;
  }

  componentWillUnmount() {
    cancelAnimationFrame(this.interval);
  }

  getStyle() {
    const TRANSITION_PROPERTY = 'fill';
    const TRANSITION_TIMING_FUNCTION = 'ease-in';

    return {
      transition: `${TRANSITION_PROPERTY} ${this.props.animation.duration}ms ${TRANSITION_TIMING_FUNCTION}`
    };
  }

  tick() {
    const ONE_SECOND = 1000;
    const COUNT_OF_FRAME = 60;
    this.setState({
      milliseconds: this.state.milliseconds + (ONE_SECOND / COUNT_OF_FRAME)
    });
    requestAnimationFrame(this.tick.bind(this));
  }

  renderLabel() {
    const {
      baseLine,
      labels,
      width,
      x,
      y
    } = this.props;

    if (labels.disable) {
      return null;
    }

    const xMiddleOfBar = x + (width / 2);

    let yText, dy;
    if (y < baseLine) {
      yText = this.state.y;
      dy = '-.6em';
    } else {
      yText = this.state.height + this.state.y;
      dy = '1.2em';
    }

    return (
      <text
        fill={labels.color}
        fontSize={labels.fontSize}
        opacity={this.state.opacity}
        textAnchor="middle"
        x={xMiddleOfBar}
        y={yText}
        dy={dy}
      >{this.state.value.toFixed(labels.digitsAfterPoint)}</text>
    );
  }

  render() {
    const {
      animation,
      fill,
      height,
      width,
      value,
      x,
      y
    } = this.props;

    const OPACITY_FINAL = 1;
    const MAX_T = 1;

    const t = Math.min(1, this.state.milliseconds / animation.duration);
    const timingFunction = ease.easePolyIn(t, MAX_T);

    this.state.height = this.state.heightCache + ((height - this.state.heightCache) * timingFunction);
    this.state.y = this.state.yCache + ((y - this.state.yCache) * timingFunction);
    this.state.value = this.state.valueCache + ((value - this.state.valueCache) * timingFunction);

    // fade in labels on init render
    this.state.opacity = this.state.opacity + ((OPACITY_FINAL - this.state.opacity) * timingFunction);

    return (
      <g>
        {this.renderLabel()}
        <rect
          fill={fill}
          height={this.state.height}
          style={this.getStyle()}
          width={width}
          x={x}
          y={this.state.y}
        />
      </g>
    );
  }
}

export default Bar;
