import React, { PropTypes, Component } from 'react';
import Bar from './bar';

class Bars extends Component {

  static propTypes = {
    animation: PropTypes.shape({
      duration: PropTypes.number.isRequired
    }).isRequired,
    baseLine: PropTypes.number.isRequired,
    colorScale: PropTypes.func.isRequired,
    data: PropTypes.arrayOf(PropTypes.object).isRequired,
    dx: PropTypes.number.isRequired,
    height: PropTypes.number.isRequired,
    labels: PropTypes.shape({
      color: PropTypes.string.isRequired,
      digitsAfterPoint: PropTypes.number.isRequired,
      disable: PropTypes.bool.isRequired,
      fontSize: PropTypes.string.isRequired
    }).isRequired,
    xScale: PropTypes.func.isRequired,
    yScale: PropTypes.func.isRequired
  };

  getStyle() {
    return { transform: `translate(${this.props.dx}px,0px)` };
  }

  render() {
    const {
      animation,
      baseLine,
      colorScale,
      data,
      height,
      labels,
      xScale,
      yScale
    } = this.props;

    const bars = data.map((val, i) => {
      const color     = colorScale(val.value),
            _height   = Math.abs(yScale(val.value) - yScale(baseLine)),
            width     = xScale.bandwidth(),
            x         = xScale(val.name),
            y         = height - yScale(Math.max(baseLine, val.value)),
            _baseLine = height - yScale(baseLine);

      return (
        <Bar
          animation={animation}
          baseLine={_baseLine}
          fill={color}
          height={_height}
          key={i}
          labels={labels}
          value={val.value}
          width={width}
          x={x}
          y={y}
        />
      );
    });

    return (
      <g style={this.getStyle()}>{bars}</g>
    );
  }
}

export default Bars;
