/* eslint-env mocha, jasmine  */
/* eslint-disable import/no-extraneous-dependencies */

import React from 'react';
import TestUtils from 'react-addons-test-utils';
import BarChart from './index';

describe('BarChart component', () => {
  beforeEach(function () {
    this.data = [
      {
        name: 'янв',
        value: -6.5
      },
      {
        name: 'июнь',
        value: 20
      }
    ];

    this.TestUtils = TestUtils;
    this.shallowRenderer = TestUtils.createRenderer();
    this.shallowRenderer.render(<BarChart data={this.data} />);
  });

  it('has svg container', function () {
    const instance = this.shallowRenderer.getRenderOutput();
    expect(instance.type).toBe('svg');
  });

  it('has expected className', function () {
    const instance = this.shallowRenderer.getRenderOutput();
    expect(instance.props.className).toBe('r3-barchart');
  });

  it('show x and y axes by default', function () {
    const renderedComponent = TestUtils.renderIntoDocument(<BarChart data={this.data} />);
    const x = TestUtils.scryRenderedDOMComponentsWithClass(renderedComponent, 'r3-barchart-yaxis');
    const y = TestUtils.scryRenderedDOMComponentsWithClass(renderedComponent, 'r3-barchart-xaxis');
    expect(x.length).toBe(1);
    expect(y.length).toBe(1);
  });

  it('hide x & y axes if disable = true', function () {
    const renderedComponent = TestUtils.renderIntoDocument(
      <BarChart data={this.data} xAxis={{ disable: true }} yAxis={{ disable: true }} />
    );
    const x = TestUtils.scryRenderedDOMComponentsWithClass(renderedComponent, 'r3-barchart-yaxis');
    const y = TestUtils.scryRenderedDOMComponentsWithClass(renderedComponent, 'r3-barchart-xaxis');
    expect(x.length).toBe(0);
    expect(y.length).toBe(0);
  });
});
