import React, {PropTypes, Component} from 'react';

const XAxisPropTypesShape = PropTypes.shape({
  className: PropTypes.string,
  dx: PropTypes.number,
  dy: PropTypes.number,
  disable: PropTypes.bool,
  fontColor: PropTypes.string,
  fontSize: PropTypes.number,
  label: PropTypes.string,
  labelColor: PropTypes.string,
  labelSize: PropTypes.number,
  lineColor: PropTypes.string,
  lineWidth: PropTypes.number,
  tickColor: PropTypes.string,
  tickCount: PropTypes.number,
  tickShift: PropTypes.number,
  tickWidth: PropTypes.number
});

const XAxisDefault = {
  className: 'r3-xaxis',
  dx: 0,
  dy: 0,
  disable: false,
  fontColor: '#333',
  fontSize: 12,
  label: '',
  labelColor: '#333',
  labelSize: 12,
  lineColor: '#666',
  lineWidth: 1,
  tickColor: '#666',
  tickCount: 5,
  tickShift: 0,
  tickWidth: 1
};

class XAxis extends Component {

  static propTypes = {
    ticks: PropTypes.array.isRequired,
    xScale: PropTypes.func.isRequired,
    className: PropTypes.string,
    dx: PropTypes.number,
    dy: PropTypes.number,
    disable: PropTypes.bool,
    fontColor: PropTypes.string,
    fontSize: PropTypes.number,
    label: PropTypes.string,
    labelColor: PropTypes.string,
    labelSize: PropTypes.number,
    lineColor: PropTypes.string,
    lineWidth: PropTypes.number,
    tickColor: PropTypes.string,
    tickCount: PropTypes.number,
    tickShift: PropTypes.number,
    tickWidth: PropTypes.number
  };

  static defaultProps = XAxisDefault;

  renderLabel() {
    const { label, labelColor, labelSize, width} = this.props;
    if (label) {
      return (<text x={width/2}
                    y="35"
                    fontSize={labelSize}
                    fill={labelColor}
                    textAnchor="middle">{label}</text>)
    } else {
      return null
    }
  }

  render() {
    const { className,
            disable,
            dx,
            dy,
            fontColor,
            fontSize,
            lineColor,
            lineWidth,
            tickColor,
            ticks,
            tickShift,
            tickWidth,
            width,
            xScale } = this.props;

    const texts = ticks.map((val, i) => {

      const x = xScale(val);

      return (
        <g transform={`translate(${x + tickShift}, 0)`} key={i}>
          <line x2="0"
                y2="7"
                strokeWidth={tickWidth}
                stroke={tickColor}/>
          <text x="0"
                y="9"
                textAnchor="middle"
                dy=".71em"
                fill={fontColor}
                fontSize={fontSize}>{val}
          </text>
        </g>
      )
    });

    if(disable){
      return null
    } else {
      return (
        <g transform={`translate(${dx}, ${dy})`}
           className={className}>
          <path d={`M0,0 V${lineWidth} H${width} V0`}
                fill={lineColor}/>
          {texts}
          {this.renderLabel()}
        </g>
      );
    }
  }
}

export { XAxisDefault, XAxisPropTypesShape };

export default XAxis;
