import React, {PropTypes, Component} from 'react';

const YAxisPropTypesShape = PropTypes.shape({
  className: PropTypes.string,
  dx: PropTypes.number,
  disable: PropTypes.bool,
  fontColor: PropTypes.string,
  fontSize: PropTypes.number,
  label: PropTypes.string,
  labelColor: PropTypes.string,
  labelSize: PropTypes.number,
  lineColor: PropTypes.string,
  lineWidth: PropTypes.number,
  tickColor: PropTypes.string,
  tickCount: PropTypes.number,
  tickShift: PropTypes.number,
  tickWidth: PropTypes.number
});

const YAxisDefault = {
  className: 'r3-yaxis',
  dx: 0,
  disable: false,
  fontColor: '#333',
  fontSize: 12,
  label: '',
  labelColor: '#333',
  labelSize: 12,
  lineColor: '#666',
  lineWidth: 1,
  tickColor: '#666',
  tickCount: 5,
  tickShift: 0,
  tickWidth: 1
};

class YAxis extends Component {

  static propTypes = {
    yScale: PropTypes.func.isRequired,
    height: PropTypes.number,
    className: PropTypes.string,
    dx: PropTypes.number,
    disable: PropTypes.bool,
    fontColor: PropTypes.string,
    fontSize: PropTypes.number,
    label: PropTypes.string,
    labelColor: PropTypes.string,
    labelSize: PropTypes.number,
    labelText: PropTypes.string,
    lineColor: PropTypes.string,
    lineWidth: PropTypes.number,
    tickColor: PropTypes.string,
    tickCount: PropTypes.number,
    tickShift: PropTypes.number,
    tickWidth: PropTypes.number
  };

  static defaultProps = YAxisDefault;

  renderLabel() {
    const { label, labelColor, labelSize} = this.props;
    if (label) {
      return (<text x="0"
                    y="1.2em"
                    fontSize={labelSize}
                    fill={labelColor}
                    transform="rotate(-90)"
                    textAnchor="end">{label}</text>)
    } else {
      return null
    }
  }

  render() {
    const { className,
            dx,
            disable,
            fontColor,
            fontSize,
            height,
            lineColor,
            lineWidth,
            tickShift,
            tickColor,
            ticks,
            tickWidth,
            yScale } = this.props;

    const texts = ticks.map((val, i) => {

      const y = yScale(val);

      return (
        <g className="tick"
           key={i}
           transform={`translate(0, ${height - y - tickShift})`}>
          <line strokeWidth={tickWidth}
                stroke={tickColor}
                x2="-6"
                y2="0"/>
          <text alignmentBaseline="middle"
                fill={fontColor}
                fontSize={fontSize}
                textAnchor="end"
                x="-9">{val}</text>
        </g>
      )
    });

    if(disable){
      return null
    } else {
      return (
        <g transform={`translate(${dx}, 0)`}
           className={className}>
          <path d={`M${lineWidth},0 H0 V${height} H${lineWidth}`}
                fill={lineColor}/>
          {texts}
          {this.renderLabel()}
        </g>
      );
    }
  }
}

export { YAxisDefault, YAxisPropTypesShape };

export default YAxis;

